--[[
	Gypsy_BuffFrame.lua
	version 9
	02.25.2006
	
	Gypsymod Buffbar - Reborn.  With new display modes and features.
	Works with Gypsymod shell v.90(barebones support,no options menu)
	and beta v1.00b(fully functional options menu), or without the shell as a standalone.

	Originally created by mondinga, and enhanced by me.  All credit must go to him for this
	great buffbar.
]]
currentversion = 9.4;

-- ** DEFAULT SETTINGS ** --
-- Default buff bar lock setting
Gypsy_DefaultLockBuffBar = 0;
-- Buff bar enable default
Gypsy_DefaultShowBuffFrame = 1;
-- Show duration default
Gypsy_DefaultShowDuration = 1;

-- ** GENERAL VARIABLES ** --

-- Base heights for both the capsule and the art frame to show on top of the calculated height
Gypsy_BuffFrameCapsuleArtHeader = 10;
Gypsy_BuffFrameCapsuleHeader = 28;
-- Amount to increment and decrement the height for each buff
Gypsy_BuffFrameCapsuleInterval = 39;
-- Save the original quest timer frame OnHide function
Gypsy_OriginalQuestTimerFrame_OnHide = QuestTimerFrame_OnHide;
-- Initialize array for effect data
Gypsy_Effects = {};
MBB_OPTIONS   = {};
options = {};

-- ** CAPSULE FUNCTIONS ** --
function Gypsy_BuffFrameCapsuleOnLoad ()
	-- Setup the colors for our capsule
	Gypsy_BuffFrameCapsuleArt:SetBackdropBorderColor(0, 0, 0);
	Gypsy_BuffFrameCapsuleArt:SetBackdropColor(0, 0, 0);
	-- Register for variable loading to run our configuration registrations from
	this:RegisterEvent("VARIABLES_LOADED");
end

function Gypsy_BuffFrameCapsuleOnEvent (event)
	if (event == "VARIABLES_LOADED") then

		player = UnitName("player").." - "..GetRealmName();

		if (not MBB_OPTIONS[player]) then
	    		MBB_OPTIONS[player] = {
				["enabled"]         = 1;
				["flashtime"]       = 5;
				["lockbuffbar"]     = 0;
				["showduration"]    = 1;
				["showdebufftypes"] = 1;
				["iconmode"]        = 0;
				["displaymode"]     = 0;
				["growbuffsup"]     = 0;
				["version"]	    = currentversion;
				["paddingup"]       = 31;
				["paddingdown"]     = -31;
				["scale"]	    = 100;
				["maxlength"]       = 24;
				["class"]           = UnitClass("player");
				["showdispellable"] = 1;
				["debufftypetextcolor"]= "greenred";
				["backdroptransparency"]= 50;
				["blackbackdrop"]   = 0;
				["subnames"]				= 0;
	    		}
			DEFAULT_CHAT_FRAME:AddMessage("Buffbar options set to defaults.");
		end

		options = MBB_OPTIONS[player];

		if (options.version == nil or options.version < currentversion) then
			options.version = currentversion;

			if (options.scale == nil or options.scale < 60) then
				options.scale = 100;
			end
			if (options.maxlength == nil) then
				options.maxlength = 24;
			end
			if (options.class == nil) then
				options.class = UnitClass("player");
			end
			if (options.showdispellable == nil) then
				options.showdispellable = 1;
			end
			if (options.debufftypetextcolor == nil) then
				options.debufftypetextcolor = "greenred";
			end
			if (options.backdroptransparency == nil) then
				options.backdroptransparency = 50;
			end
			if (options.blackbackdrop == nil) then
				options.blackbackdrop = 0;
			end
			if (options.subnames == nil) then
				options.subnames = 0;
			end
		end

		Gypsy_ShowDuration  = options.showduration;
		Gypsy_ShowBuffFrame = options.enabled;
		Gypsy_LockBuffBar   = options.lockbuffbar;

		if (options.displaymode == 0) then
			BuffFrame:Hide();
			TempEnchant1:Show();
			TempEnchant2:Show();
		elseif (options.displaymode == 1) then
			BuffFrame:Show();
		end

		BUFF_WARNING_TIME = options.flashtime;

		SlashCmdList["MBB_SLASHCOMMANDS"] = mbb_SlashCommands;
		SLASH_MBB_SLASHCOMMANDS1 = "/mbb";
		SLASH_MBB_SLASHCOMMANDS2 = "/mbbb";


		-- Register with GypsyMod if the Shell is present, otherwise make stand-alone registrations
		if (GYPSY_SHELL == 1 and GYPSY_VERSION == "1.00b") then
			--load shell menu
			MBB_LoadGypsySetup();

		elseif (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
			-- Register with GypsyMod shell v .90
			Gypsy_RegisterOption(150, "category", nil, nil, nil, "BuffBar", "Settings for the player effects display.");
			Gypsy_RegisterOption(151, "check", Gypsy_ShowBuffFrame, "ShowBuffFrame", Gypsy_BuffFrameUpdateAll, "Enable BuffBar", "Click to toggle between the default\nbuff frame and the GypsyMod frame.");
			Gypsy_RegisterOption(152, "check", Gypsy_ShowDuration, "ShowDuration", Gypsy_BuffFrameUpdateAll, "Show Durations", "Display durations for effects in\ntext below their name.");
			Gypsy_RegisterOption(153, "button", nil, nil, Gypsy_BuffFrameReset, "Reset", "Reset buff frame options and position.");
			--dont know why this doesnt add an option to the shell menu.... hmm oh well
			--Gypsy_RegisterOption(222, "check", mbb_Displaymode, "AlternativeDisplay", nil, "Activate/DeActivate alternative display mode.", "Turn alternative display mode on/off.");
			SlashCmdList["GYPSY_DISPLAYTOGGLE"] = mbb_DisplayToggle;
			SLASH_GYPSY_DISPLAYTOGGLE1 = "/mbbd";
			SLASH_GYPSY_DISPLAYTOGGLE2 = "/mbbdisplay";
		else

			-- Register slash commands
			SlashCmdList["GYPSY_SHOWBUFFBAR"] = Gypsy_ShowBuffBarSlashHandler;
			SLASH_GYPSY_SHOWBUFFBAR1 = "/buffbarenable";
			SLASH_GYPSY_SHOWBUFFBAR2 = "/bbenable";
			SlashCmdList["GYPSY_SHOWDURATION"] = Gypsy_ShowDurationSlashHandler;			
			SLASH_GYPSY_SHOWDURATION1 = "/buffbarshowduration";
			SLASH_GYPSY_SHOWDURATION2 = "/bbshowduration";			
			SlashCmdList["GYPSY_RESETBUFFBAR"] = Gypsy_ResetBuffBarSlashHandler;			
			SLASH_GYPSY_RESETBUFFBAR1 = "/buffbarreset";
			SLASH_GYPSY_RESETBUFFBAR2 = "/bbreset";	
			SlashCmdList["GYPSY_LOCKBUFFBAR"] = Gypsy_LockBuffBarSlashHandler;
			SLASH_GYPSY_LOCKBUFFBAR1 = "/buffbarlock";
			SLASH_GYPSY_LOCKBUFFBAR2 = "/bblock";
			-- Special slash command to display listing of all other slash commands
			SlashCmdList["GYPSY_BUFFBAR"] = Gypsy_BuffBarSlashHandler;
			SLASH_GYPSY_BUFFBAR1 = "/buffbar";
			SLASH_GYPSY_BUFFBAR2 = "/bb";

			SlashCmdList["GYPSY_DISPLAYTOGGLE"] = mbb_DisplayToggle;
			SLASH_GYPSY_DISPLAYTOGGLE1 = "/mbbd";
			SLASH_GYPSY_DISPLAYTOGGLE2 = "/mbbdisplay";

			SlashCmdList["MBB_SLASHCOMMANDS"] = mbb_SlashCommands;
			SLASH_MBB_SLASHCOMMANDS1 = "/mbb";
			SLASH_MBB_SLASHCOMMANDS2 = "/mbbb";

		end
		-- Initial buff frame setting update
		Gypsy_BuffFrameUpdateAll();
		return;
	end
end


function mbb_reset()
	--reset options to defaults
	    	MBB_OPTIONS[player] = {
			["enabled"]         = 1;
			["flashtime"]       = 5;
			["lockbuffbar"]     = 0;
			["showduration"]    = 1;
			["showdebufftypes"] = 1;
			["iconmode"]        = 0;
			["displaymode"]     = 0;
			["growbuffsup"]     = 0;
			["version"]	    = currentversion;
			["paddingup"]       = 31;
			["paddingdown"]     = -31;
			["scale"]	    = 100;
			["maxlength"]       = 24;
			["class"]           = UnitClass("player");
			["showdispellable"] = 1;
			["debufftypetextcolor"]= "greenred";
			["backdroptransparency"]= 50;
			["blackbackdrop"]   = 0;
			["subnames"]				= 0;
	    	}
		options = MBB_OPTIONS[player];
end	

function mbb_DisplayToggle(msg)
	if (options.displaymode == 0) then
		options.displaymode = 1;
		Gypsy_BuffFrameUpdateAll();
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Turning Alternate Display mode ON.");
	else
		options.displaymode = 0;
		Gypsy_BuffFrameUpdateAll();
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Turning Alternate Display mode OFF.");
	end
end	

-- Main function to change the buff frame height according to number of effects
function Gypsy_BuffFrameCapsuleOnUpdate ()
	-- Variable to hold number of buffs shown
	local count = 0;
	-- Check each buff button for visibility and add to the count variable if it is
	for i=0, 23 do
		local button = getglobal("Gypsy_BuffButton"..i);
		if (button:IsVisible()) then
			count = count + 1;
		end
	end
	-- Calculate our height adjustment
	local heightAdjustment = Gypsy_BuffFrameCapsuleInterval * count;

	-- Always display one buff's worth of height
	if (heightAdjustment == 0) then
		heightAdjustment = Gypsy_BuffFrameCapsuleInterval;
	end
	

	-- Calculate the height to make each frame
	local capsuleHeight = Gypsy_BuffFrameCapsuleHeader + heightAdjustment;
	local capsuleArtHeight = Gypsy_BuffFrameCapsuleArtHeader + heightAdjustment;

	-- Set heights
	if (options.growbuffsup == 0) then
		Gypsy_BuffFrameCapsule:SetHeight(capsuleHeight);
		Gypsy_BuffFrameCapsuleArt:SetHeight(capsuleArtHeight);
	else
	--growing buffs upwards, need to set a large capsule so buffs scroll upwards properly
		Gypsy_BuffFrameCapsule:SetHeight(410);
		Gypsy_BuffFrameCapsuleArt:SetHeight(400);
	end
	--set the scale of the buffs/capsule frame
	local bfs = options.scale / 100;
	Gypsy_BuffFrame:SetScale(bfs);
	Gypsy_BuffFrameCapsule:SetScale(bfs);
end

function Gypsy_ToggleBuffFrameCapsule ()
	-- If the Shell is present, update the local lock setting
	if (GYPSY_SHELL ~= nil and GYPSY_VERSION == "1.00b") then
		if (GYPSY_HOTBAR ~= nil) then
			Gypsy_LockBuffBar = Gypsy_Capsule.locked;
		end
	elseif (GYPSY_SHELL ~= nil and GYPSY_VERSION == "0.90") then
		Gypsy_LockBuffBar = GYPSY_LOCKALL;
	end

	-- Then, if the buff bar is not locked, go ahead and show/hide or drag the capsule as needed, or if it is locked, hide the capsule so it can't be dragged
	if (Gypsy_LockBuffBar == 0) then
		-- Make sure the capsule is showing, it might have been previously hidden
		if (not Gypsy_BuffFrameCapsule:IsVisible()) then
			Gypsy_BuffFrameCapsule:Show();
		end
		-- Quick function to check if the mouse is anywhere over our capsule
		if (MouseIsOver(Gypsy_BuffFrameCapsule)) then
			Gypsy_BuffFrameCapsuleArt:Show();
		else
			Gypsy_BuffFrameCapsuleArt:Hide();
		end
	else
		Gypsy_BuffFrameCapsule:Hide();
	end
end

-- ** BUFF FRAME FUNCTIONS ** --

function Gypsy_BuffFrameUpdateAll ()
	-- Get value from the shell table if it's present
	if (GYPSY_SHELL ~= nil and GYPSY_VERSION == "1.00b") then
		Gypsy_ShowBuffFrame = options.enabled;
	elseif (GYPSY_SHELL ~= nil and GYPSY_VERSION == "0.90") then
		if (Gypsy_RetrieveOption ~= nil) then
			Gypsy_ShowBuffFrame = Gypsy_RetrieveOption(151)[GYPSY_VALUE];
		else
			Gypsy_ShowBuffFrame = options.enabled;
		end
	end

	-- Check to see if we're showing our buff frame or the default, and show/hide each as needed, and also move the durability and quest timer frames
	if (Gypsy_ShowBuffFrame == 1) then

		if (options.displaymode == 0) then
			BuffFrame:Hide();
			Gypsy_BuffFrame:Show();
			Gypsy_BuffFrameCapsule:Show();
			mbb_filter = "HELPFUL|HARMFUL";
		elseif (options.displaymode == 1) then
			--displaying debuffs with gypsy and buffs with default buffframe
			BuffFrame:Show();
			mbb_filter = "HARMFUL";
		end

		-- Place where the default buff frame would be to the right of the minimap
		DurabilityFrame:ClearAllPoints();
		DurabilityFrame:SetPoint("TOPRIGHT", "MinimapCluster", "TOPLEFT", -10, -10);
		QuestTimerFrame:ClearAllPoints();
		QuestTimerFrame:SetPoint("TOPRIGHT", "MinimapCluster", "TOPLEFT", 10, -10);

		if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
			Gypsy_Option152:Enable();
			Gypsy_Option152Text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		end

	else

		BuffFrame:Show();
		Gypsy_BuffFrame:Hide();
		Gypsy_BuffFrameCapsule:Hide();
		-- Default spots
		DurabilityFrame:ClearAllPoints();
		DurabilityFrame:SetPoint("TOP", "MinimapCluster", "BOTTOM", 40, 15);
		QuestTimerFrame:ClearAllPoints();
		QuestTimerFrame:SetPoint("TOP", "MinimapCluster", "BOTTOM", 10, 0);

		if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
			Gypsy_Option152:Disable();
			Gypsy_Option152Text:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		end	

	end

	-- Update each buff frame to account for settings changes
	for i=0, 23 do
		local button = getglobal("Gypsy_BuffButton"..i);
		Gypsy_BuffButtonLoadAny(button);
		mbb_changelayout(button, mbb_filter);
	end
end

-- Replacing the original quest timer frame OnHide function, which moves the durability frame
function QuestTimerFrame_OnHide()
	-- Get show buff frame setting
	if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
		if (Gypsy_RetrieveOption ~= nil) then
			if (Gypsy_RetrieveOption(151) ~= nil) then
				Gypsy_ShowBuffFrame = Gypsy_RetrieveOption(151)[GYPSY_VALUE];
			end
		end
	end

	DurabilityFrame:ClearAllPoints();
	-- If the buff frame is showing, move the durability frame to our spot, else run the original function to use the default spot
	if (options.enabled == 1) then		
		DurabilityFrame:SetPoint("TOPRIGHT", "MinimapCluster", "TOPLEFT", -10, -10);
	else
		Gypsy_OriginalQuestTimerFrame_OnHide();
	end
end

function Gypsy_BuffFrameReset ()
	-- Change values for both Shell and stand-alone mode
	if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
		-- Set the GypsyMod values to default
		Gypsy_UpdateValue(151, Gypsy_DefaultShowBuffFrame);
		Gypsy_UpdateValue(152, Gypsy_DefaultShowDuration);
		-- Set the check button(s) to default state
		Gypsy_Option151:SetChecked(Gypsy_DefaultShowBuffFrame);
		Gypsy_Option152:SetChecked(Gypsy_DefaultShowDuration);
	else
		options.enabled = Gypsy_DefaultShowBuffFrame;
		options.showduration = Gypsy_DefaultShowDuration;
	end

	-- Move the buff frame to it's default position
	Gypsy_BuffFrameCapsule:ClearAllPoints();
	Gypsy_BuffFrameCapsule:SetPoint("TOPRIGHT", "MinimapCluster", "BOTTOMRIGHT", 0, 0);
	-- Run the update all function to reflect changes
	Gypsy_BuffFrameUpdateAll();
end

function Gypsy_BuffFrameOnUpdate(elapsed)
	if ( BuffFrameUpdateTime > 0 ) then
		BuffFrameUpdateTime = BuffFrameUpdateTime - elapsed;
	else
		BuffFrameUpdateTime = BuffFrameUpdateTime + TOOLTIP_UPDATE_TIME;
	end

	BuffFrameFlashTime = BuffFrameFlashTime - elapsed;

	if ( BuffFrameFlashTime < 0 ) then
		local overtime = -BuffFrameFlashTime;
		if ( BuffFrameFlashState == 1 ) then
			BuffFrameFlashState = 0;
			BuffFrameFlashTime = BUFF_FLASH_TIME_OFF;
		else
			BuffFrameFlashState = 1;
			BuffFrameFlashTime = BUFF_FLASH_TIME_ON;
		end
		if ( overtime < BuffFrameFlashTime ) then
			BuffFrameFlashTime = BuffFrameFlashTime - overtime;
		end
	end
	-- Check whether the mouse is over our capsule to show or hide it	
	Gypsy_ToggleBuffFrameCapsule();	
end

-- ** BUFF BUTTON FUNCTIONS ** --
function Gypsy_BuffButtonUpdate()
if (options.enabled == 1) then
	if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
		if (Gypsy_RetrieveOption ~= nil) then
			if (Gypsy_RetrieveOption(152) ~= nil) then
				Gypsy_ShowDuration = Gypsy_RetrieveOption(152)[GYPSY_VALUE];
			end
		end
	end

	if (options.displaymode == 0) then
		--standard gypsy buff display
		mbb_filter = "HELPFUL|HARMFUL";

		--hide the default ui buffs
		BuffFrame:Hide();

		--show default ui weapon enchants
		TempEnchant1:Show();
		TempEnchant2:Show();

	elseif (options.displaymode == 1) then
		--displaying debuffs with gypsy and buffs with default buffframe
        	mbb_filter = "HARMFUL";

		--show default ui buffs
		BuffFrame:Show();

		--hide 8 debuff slots
		BuffButton16:Hide();
		BuffButton17:Hide();
		BuffButton18:Hide();
		BuffButton19:Hide();
		BuffButton20:Hide();
		BuffButton21:Hide();
		BuffButton22:Hide();
		BuffButton23:Hide();
		
	end


	local buffIndex, untilCancelled = GetPlayerBuff(this:GetID(), mbb_filter);

	this.buffIndex = buffIndex;
	this.untilCancelled = untilCancelled;

	if ( buffIndex < 0 ) then		
		this:Hide();
		return;
	else
		this:SetAlpha(1.0);
		this:Show();
	end

	local icon = getglobal(this:GetName().."Icon");
	icon:SetTexture(GetPlayerBuffTexture(buffIndex, mbb_filter));

	-- Set the number of applications of an aura if its a debuff
	local buffCount = getglobal(this:GetName().."Count");
	local count = GetPlayerBuffApplications(buffIndex);

	-- Since 1.9 also Buffs can have stacks
	if ( count > 1 ) then
		buffCount:SetText(count);
		buffCount:Show();
	else
		buffCount:Hide();
	end

	if (GameTooltip:IsOwned(this)) then
		GameTooltip:SetPlayerBuff(buffIndex, mbb_filter);
	end
	
	-- Get the buff description and display it
	local buffName = Gypsy_GetBuffName("player", this:GetID(), mbb_filter);
	if (buffName) then
		getglobal(this:GetName() .. "DescribeText"):SetText(buffName);
	end
	Gypsy_Effects[this:GetID()] = nil;
end
end


function mbb_changelayout(button, filter)
	local buffIndex, untilCancelled = GetPlayerBuff(button:GetID(), filter);

	button.buffIndex = buffIndex;
	button.untilCancelled = untilCancelled;

	local icon = getglobal(button:GetName().."Icon");
	icon:SetTexture(GetPlayerBuffTexture(buffIndex, filter));

	local thisbuff = getglobal(button:GetName());
	local bufficonframe = getglobal(button:GetName().."Buff");
	local debufficonframe = getglobal(button:GetName().."Debuff");
	local countframe = getglobal(button:GetName().."Count");

	if (options.growbuffsup == 1) then
		--buffs scrolling upwards instead of downwards

		if(options.iconmode == 0) then
			--if icons on left

			if (icon ~= nil) then
				icon:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", -28, -2);
			end
			if (bufficonframe ~= nil) then
				bufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", -29, -1);
			end
 			if (debufficonframe ~= nil) then
				debufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", -29, -1);
			end

			if (countframe ~= nil) then
				countframe:ClearAllPoints();
				countframe:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", -1, 1);
			end

			Gypsy_BuffFrame:ClearAllPoints();
			Gypsy_BuffButton0:SetPoint("TOPLEFT", "Gypsy_BuffFrameCapsule", "TOPLEFT", 36, -377);
			Gypsy_BuffButton1:SetPoint("TOPLEFT", "Gypsy_BuffButton0", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton2:SetPoint("TOPLEFT", "Gypsy_BuffButton1", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton3:SetPoint("TOPLEFT", "Gypsy_BuffButton2", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton4:SetPoint("TOPLEFT", "Gypsy_BuffButton3", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton5:SetPoint("TOPLEFT", "Gypsy_BuffButton4", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton6:SetPoint("TOPLEFT", "Gypsy_BuffButton5", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton7:SetPoint("TOPLEFT", "Gypsy_BuffButton6", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton8:SetPoint("TOPLEFT", "Gypsy_BuffButton7", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton9:SetPoint("TOPLEFT", "Gypsy_BuffButton8", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton10:SetPoint("TOPLEFT", "Gypsy_BuffButton9", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton11:SetPoint("TOPLEFT", "Gypsy_BuffButton10", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton12:SetPoint("TOPLEFT", "Gypsy_BuffButton11", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton13:SetPoint("TOPLEFT", "Gypsy_BuffButton12", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton14:SetPoint("TOPLEFT", "Gypsy_BuffButton13", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton15:SetPoint("TOPLEFT", "Gypsy_BuffButton14", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton16:SetPoint("TOPLEFT", "Gypsy_BuffButton15", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton17:SetPoint("TOPLEFT", "Gypsy_BuffButton16", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton18:SetPoint("TOPLEFT", "Gypsy_BuffButton17", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton19:SetPoint("TOPLEFT", "Gypsy_BuffButton18", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton20:SetPoint("TOPLEFT", "Gypsy_BuffButton19", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton21:SetPoint("TOPLEFT", "Gypsy_BuffButton20", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton22:SetPoint("TOPLEFT", "Gypsy_BuffButton21", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton23:SetPoint("TOPLEFT", "Gypsy_BuffButton22", "TOPLEFT", 0, options.paddingup);
		else
			--icons on right

			if (icon ~= nil) then
				icon:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", 153, -2);
			end
			if (bufficonframe ~= nil) then
				bufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", 152, -1);
			end
 			if (debufficonframe ~= nil) then
				debufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", 152, -1);
			end
			if (countframe ~= nil) then
				countframe:ClearAllPoints();
				countframe:SetPoint("BOTTOMLEFT", icon, "BOTTOMLEFT", 1, 1);
			end

			Gypsy_BuffFrame:ClearAllPoints();
			Gypsy_BuffButton0:SetPoint("TOPLEFT", "Gypsy_BuffFrameCapsule", "TOPLEFT", 10, -377);
			Gypsy_BuffButton1:SetPoint("TOPLEFT", "Gypsy_BuffButton0", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton2:SetPoint("TOPLEFT", "Gypsy_BuffButton1", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton3:SetPoint("TOPLEFT", "Gypsy_BuffButton2", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton4:SetPoint("TOPLEFT", "Gypsy_BuffButton3", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton5:SetPoint("TOPLEFT", "Gypsy_BuffButton4", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton6:SetPoint("TOPLEFT", "Gypsy_BuffButton5", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton7:SetPoint("TOPLEFT", "Gypsy_BuffButton6", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton8:SetPoint("TOPLEFT", "Gypsy_BuffButton7", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton9:SetPoint("TOPLEFT", "Gypsy_BuffButton8", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton10:SetPoint("TOPLEFT", "Gypsy_BuffButton9", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton11:SetPoint("TOPLEFT", "Gypsy_BuffButton10", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton12:SetPoint("TOPLEFT", "Gypsy_BuffButton11", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton13:SetPoint("TOPLEFT", "Gypsy_BuffButton12", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton14:SetPoint("TOPLEFT", "Gypsy_BuffButton13", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton15:SetPoint("TOPLEFT", "Gypsy_BuffButton14", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton16:SetPoint("TOPLEFT", "Gypsy_BuffButton15", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton17:SetPoint("TOPLEFT", "Gypsy_BuffButton16", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton18:SetPoint("TOPLEFT", "Gypsy_BuffButton17", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton19:SetPoint("TOPLEFT", "Gypsy_BuffButton18", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton20:SetPoint("TOPLEFT", "Gypsy_BuffButton19", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton21:SetPoint("TOPLEFT", "Gypsy_BuffButton20", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton22:SetPoint("TOPLEFT", "Gypsy_BuffButton21", "TOPLEFT", 0, options.paddingup);
			Gypsy_BuffButton23:SetPoint("TOPLEFT", "Gypsy_BuffButton22", "TOPLEFT", 0, options.paddingup);
		end
	elseif (options.growbuffsup == 0) then
		--buffs scrolling downwards
		
		if (options.iconmode == 0) then
			--icons on left

			if (icon ~= nil) then
				icon:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", -28, -2);
			end
			if (bufficonframe ~= nil) then
				bufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", -29, -1);
			end
 			if (debufficonframe ~= nil) then
				debufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", -29, -1);
			end
			if (countframe ~= nil) then
				countframe:ClearAllPoints();
				countframe:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", -1, 1);
			end

			Gypsy_BuffFrame:ClearAllPoints();
			Gypsy_BuffButton0:SetPoint("TOPLEFT", "Gypsy_BuffFrameCapsule", "TOPLEFT", 38, -20);
			Gypsy_BuffButton1:SetPoint("TOPLEFT", "Gypsy_BuffButton0", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton2:SetPoint("TOPLEFT", "Gypsy_BuffButton1", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton3:SetPoint("TOPLEFT", "Gypsy_BuffButton2", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton4:SetPoint("TOPLEFT", "Gypsy_BuffButton3", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton5:SetPoint("TOPLEFT", "Gypsy_BuffButton4", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton6:SetPoint("TOPLEFT", "Gypsy_BuffButton5", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton7:SetPoint("TOPLEFT", "Gypsy_BuffButton6", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton8:SetPoint("TOPLEFT", "Gypsy_BuffButton7", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton9:SetPoint("TOPLEFT", "Gypsy_BuffButton8", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton10:SetPoint("TOPLEFT", "Gypsy_BuffButton9", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton11:SetPoint("TOPLEFT", "Gypsy_BuffButton10", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton12:SetPoint("TOPLEFT", "Gypsy_BuffButton11", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton13:SetPoint("TOPLEFT", "Gypsy_BuffButton12", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton14:SetPoint("TOPLEFT", "Gypsy_BuffButton13", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton15:SetPoint("TOPLEFT", "Gypsy_BuffButton14", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton16:SetPoint("TOPLEFT", "Gypsy_BuffButton15", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton17:SetPoint("TOPLEFT", "Gypsy_BuffButton16", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton18:SetPoint("TOPLEFT", "Gypsy_BuffButton17", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton19:SetPoint("TOPLEFT", "Gypsy_BuffButton18", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton20:SetPoint("TOPLEFT", "Gypsy_BuffButton19", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton21:SetPoint("TOPLEFT", "Gypsy_BuffButton20", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton22:SetPoint("TOPLEFT", "Gypsy_BuffButton21", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton23:SetPoint("TOPLEFT", "Gypsy_BuffButton22", "TOPLEFT", 0, options.paddingdown);

		else
			--icons on right

			if (icon ~= nil) then
				icon:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", 153, -2);
			end
			if (bufficonframe ~= nil) then
				bufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", 152, -1);
			end
 			if (debufficonframe ~= nil) then
				debufficonframe:SetPoint("TOPLEFT", thisbuff, "TOPLEFT", 152, -1);
			end
			if (countframe ~= nil) then
				countframe:ClearAllPoints();
				countframe:SetPoint("BOTTOMLEFT", icon, "BOTTOMLEFT", 1, 1);
			end

			Gypsy_BuffButton0:SetPoint("TOPLEFT", "Gypsy_BuffFrameCapsule", "TOPLEFT", 5, -20);
			Gypsy_BuffButton1:SetPoint("TOPLEFT", "Gypsy_BuffButton0", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton2:SetPoint("TOPLEFT", "Gypsy_BuffButton1", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton3:SetPoint("TOPLEFT", "Gypsy_BuffButton2", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton4:SetPoint("TOPLEFT", "Gypsy_BuffButton3", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton5:SetPoint("TOPLEFT", "Gypsy_BuffButton4", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton6:SetPoint("TOPLEFT", "Gypsy_BuffButton5", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton7:SetPoint("TOPLEFT", "Gypsy_BuffButton6", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton8:SetPoint("TOPLEFT", "Gypsy_BuffButton7", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton9:SetPoint("TOPLEFT", "Gypsy_BuffButton8", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton10:SetPoint("TOPLEFT", "Gypsy_BuffButton9", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton11:SetPoint("TOPLEFT", "Gypsy_BuffButton10", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton12:SetPoint("TOPLEFT", "Gypsy_BuffButton11", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton13:SetPoint("TOPLEFT", "Gypsy_BuffButton12", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton14:SetPoint("TOPLEFT", "Gypsy_BuffButton13", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton15:SetPoint("TOPLEFT", "Gypsy_BuffButton14", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton16:SetPoint("TOPLEFT", "Gypsy_BuffButton15", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton17:SetPoint("TOPLEFT", "Gypsy_BuffButton16", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton18:SetPoint("TOPLEFT", "Gypsy_BuffButton17", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton19:SetPoint("TOPLEFT", "Gypsy_BuffButton18", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton20:SetPoint("TOPLEFT", "Gypsy_BuffButton19", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton21:SetPoint("TOPLEFT", "Gypsy_BuffButton20", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton22:SetPoint("TOPLEFT", "Gypsy_BuffButton21", "TOPLEFT", 0, options.paddingdown);
			Gypsy_BuffButton23:SetPoint("TOPLEFT", "Gypsy_BuffButton22", "TOPLEFT", 0, options.paddingdown);
		end
	end
end

-- Duplicate functionality of the normal button update function, usable with a supplied button reference instead of from the button itself
function Gypsy_BuffButtonUpdateAny(button)
	if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
		if (Gypsy_RetrieveOption ~= nil) then
			if (Gypsy_RetrieveOption(152) ~= nil) then
				Gypsy_ShowDuration = Gypsy_RetrieveOption(152)[GYPSY_VALUE];
			end
		end
	end

	local buffIndex, untilCancelled = GetPlayerBuff(button:GetID(), button.buffFilter);

	button.buffIndex = buffIndex;
	button.untilCancelled = untilCancelled;

	if ( buffIndex < 0 ) then		
		button:Hide();
		return;
	else
		button:SetAlpha(1.0);
		button:Show();
	end

	local icon = getglobal(button:GetName().."Icon");
	icon:SetTexture(GetPlayerBuffTexture(buffIndex, button.buffFilter));

	if (GameTooltip:IsOwned(button)) then
		GameTooltip:SetPlayerBuff(buffIndex, button.buffFilter);
	end
	
	-- Get the buff description and display it
	local buffName = Gypsy_GetBuffName("player", button:GetID(), button.buffFilter);
	if (buffName) then
		getglobal(button:GetName() .. "DescribeText"):SetText(buffName);
	end
	Gypsy_Effects[button:GetID()] = nil;
end

function Gypsy_BuffButtonOnLoad()
	-- Update the duration show setting
	if (GYPSY_SHELL == 1 and GYPSY_VERSION == "0.90") then
		if (Gypsy_RetrieveOption ~= nil) then
			if (Gypsy_RetrieveOption(152) ~= nil) then
				Gypsy_ShowDuration = Gypsy_RetrieveOption(152)[GYPSY_VALUE];
			end
		end
	end

	local buffIndex, untilCancelled = GetPlayerBuff(this:GetID(), this.buffFilter);
	
	if (Gypsy_BuffIsDebuff(temp) == 1) then
		getglobal(this:GetName() .. "Debuff"):Show();
	else
		getglobal(this:GetName() .. "Debuff"):Hide();
	end
	if (untilCancelled == 1 or Gypsy_ShowDuration == 0) then
		getglobal(this:GetName() .. "DurationText"):SetText("");
	end

	Gypsy_BuffButtonUpdate();
	this:RegisterForClicks("RightButtonUp");
	this:RegisterEvent("PLAYER_AURAS_CHANGED");
	
	-- Get the buff description and display it
	local description = getglobal(this:GetName() .. "DescribeText");
	if (description) then 
		description:Show();
	end
end

-- Duplicate functionality of the normal button load function, usable with a supplied button reference instead of from the button itself
function Gypsy_BuffButtonLoadAny(button)
	-- Update the duration show setting
	Gypsy_ShowDuration = 1;

	local buffIndex, untilCancelled = GetPlayerBuff(button:GetID(), button.buffFilter);
	
	if (Gypsy_BuffIsDebuff(temp) == 1) then
		getglobal(button:GetName() .. "Debuff"):Show();
	else
		getglobal(button:GetName() .. "Debuff"):Hide();
	end
	if (untilCancelled == 1 or Gypsy_ShowDuration == 0) then
		getglobal(button:GetName() .. "DurationText"):SetText("");
	end

	Gypsy_BuffButtonUpdateAny(button);

	-- Get the buff description and display it
	local description = getglobal(button:GetName() .. "DescribeText");
	if (description) then 
		description:Show(); 
	end
end

function Gypsy_BuffButtonOnEvent(event)
	Gypsy_BuffButtonUpdate();
end

function Gypsy_BuffButtonOnUpdate()
	local buffName;
    	if (options.displaymode == 1) then
        	mbb_filter = "HARMFUL";
    	elseif (options.displaymode == 0) then
        	mbb_filter = "HELPFUL|HARMFUL";
    	end

	local buffIndex, untilCancelled = GetPlayerBuff(this:GetID(), mbb_filter);

	-- Get border and background textures
	local buffBorder = getglobal(this:GetName().."Buff");
	local debuffBorder = getglobal(this:GetName().."Debuff");
	local bdtp = options.backdroptransparency / 100;

	-- Set the correct borders/backgrounds depending on if this is a buff or debuff
	if (Gypsy_BuffIsDebuff(buffIndex) == 1) then
		debuffBorder:Show();
		buffBorder:Hide();
		this:SetBackdropBorderColor(1, 0, 0);

		if (options.blackbackdrop == 1) then
			this:SetBackdropColor(0, 0, 0, bdtp);
		else
			this:SetBackdropColor(1, 0, 0, bdtp);
		end
	else
		--this fixes the glitches when changing display modes from Classic to Debuffs Only
		if (options.displaymode == 0) then
			debuffBorder:Hide();
			buffBorder:Show();
		else
			this:Hide();
		end
		this:SetBackdropBorderColor(0, 0.75, 1);

		if (options.blackbackdrop == 1) then
			this:SetBackdropColor(0, 0, 0, bdtp);
		else
			this:SetBackdropColor(0, 0.5, 1, bdtp);
		end
	end


	if (not Gypsy_Effects[this:GetID()]) then
		buffName = Gypsy_GetBuffName("player", this:GetID());
		Gypsy_Effects[this:GetID()] = buffName;
	else
		buffName = Gypsy_Effects[this:GetID()];
	end


	local timeLeft = GetPlayerBuffTimeLeft(buffIndex);
	local buffAlphaValue;

	local buffName = Gypsy_GetBuffName("player",this:GetID(), mbb_filter);
	local buffType = mbb_GetBuffType("player",this:GetID(), mbb_filter);

	Gypsy_ShowDuration = options.showduration;

	-- clear duration/debuff type text if applicable and set spellname
	if (untilCancelled == 1) then
		getglobal(this:GetName().."DescribeText"):SetText(buffName);
		getglobal(this:GetName().."DurationText"):SetText("");
		getglobal(this:GetName().."buffTypeText"):SetText("");
		return;
	end

	--set spellname text
	if (buffName) then
		getglobal(this:GetName().."DescribeText"):SetText(buffName);
	end

	-- Set duration text
	if (Gypsy_ShowDuration == 1 and timeLeft ~= 0) then
		getglobal(this:GetName().."DurationText"):SetText(Gypsy_GetStringTime(timeLeft));
	else
		getglobal(this:GetName().."DurationText"):SetText("");
	end

	-- set debuff type text
	if (buffType ~= nil and options.showdebufftypes == 1 and Gypsy_BuffIsDebuff(buffIndex) == 1) then

		if (options.showdispellable == 1) then
			if(options.debufftypetextcolor == "greenred") then
				can1 = 0;
				can2 = 1;
				can3 = 0;
				cant1= 1;
				cant2= 0;
				cant3 = 0;
			elseif (options.debufftypetextcolor == "greenyellow") then
				can1 = 0;
				can2 = 1;
				can3 = 0;
				cant1= 1;
				cant2= 1;
				cant3 = 0;
			else
				can1 = 0;
				can2 = 1;
				can3 = 0;
				cant1= 1;
				cant2= 0;
				cant3 = 0;			
			end

			if (options.class == "Priest") then
				if(buffType == "Magic") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(can1,can2,can3);
				elseif (buffType == "Disease") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
				end
			elseif (options.class == "Paladin") then
				if(buffType == "Magic") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Disease") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Poison") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
				end
			elseif (options.class == "Shaman") then
				if(buffType == "Magic") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Poison") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Disease") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
				end
			elseif (options.class == "Mage") then
				if(buffType == "Curse") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(1,0,0);
				end
			elseif (options.class == "Druid") then
				if (buffType == "Curse") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Poison") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
				end
			elseif (options.class == "Paladin") then
				if(buffType == "Magic") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Poison") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				elseif (buffType == "Disease") then
					getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
				end
			elseif (options.class == "Warlock") then
				if(buffType == "Magic") then
					local pettype = UnitCreatureFamily("pet");
					if (pettype ~= nil and pettype == "Felhunter") then
						getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);
					else
						getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
					end
				else
					getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
				end
			else
				getglobal(this:GetName().."buffTypeText"):SetTextColor(cant1,cant2,cant3);
			end
		else
				getglobal(this:GetName().."buffTypeText"):SetTextColor(0,1,0);			
		end
		getglobal(this:GetName().."buffTypeText"):SetText(buffType);
	end


	BUFF_WARNING_TIME = options.flashtime;

	if (timeLeft < BUFF_WARNING_TIME) then
		if (BuffFrameFlashState == 1) then
			buffAlphaValue = (BUFF_FLASH_TIME_ON - BuffFrameFlashTime) / BUFF_FLASH_TIME_ON;
			buffAlphaValue = buffAlphaValue * (1 - BUFF_MIN_ALPHA) + BUFF_MIN_ALPHA;
		else
			buffAlphaValue = BuffFrameFlashTime / BUFF_FLASH_TIME_ON;
			buffAlphaValue = (buffAlphaValue * (1 - BUFF_MIN_ALPHA)) + BUFF_MIN_ALPHA;
			this:SetAlpha(BuffFrameFlashTime / BUFF_FLASH_TIME_ON);
		end
		this:SetAlpha(buffAlphaValue);
	end

	if (BuffFrameUpdateTime > 0) then
		return;
	end
	if (GameTooltip:IsOwned(this)) then
		GameTooltip:SetPlayerBuff(buffIndex);
	end
end

function Gypsy_BuffButtonOnClick()
	CancelPlayerBuff(this.buffIndex);
end

function Gypsy_BuffIsDebuff(id)
	for z=0, 23, 1 do
		local debuffIndex, debuffTemp = GetPlayerBuff(z, "HARMFUL");
		if (debuffIndex == -1) then 
			return 0; 
		end
		if (debuffIndex == id) then
			return 1;
		end
	end
	return 0;
end

function Gypsy_GetBuffName(unit, i, filter)
	if (filter == nil) then
		if(mbb_displaymode == 0) then
			filter = "HELPFUL|HARMFUL";
		else
			filter = "HARMFUL";
		end
	end

	local buffIndex, untilCancelled = GetPlayerBuff(this:GetID(), filter);
	local buff;

	if (buffIndex < 24) then
		buff = buffIndex;
		if (buff == -1) then
			buff = nil;
		end
	end

	if (buff) then
		local tooltip = BTooltip;
		tooltip:SetOwner(UIParent, "ANCHOR_NONE");
		if (unit == "player" and tooltip ) then
			tooltip:SetPlayerBuff(buffIndex);
		end
		local toolTipText = getglobal("BTooltipTextLeft1");
		if (toolTipText) then
			local name = toolTipText:GetText();
			if ( name ~= nil ) then
				if (options.subnames == 1) then
					name = string.gsub(name, "Power Word", "PW");
					name = string.gsub(name, "Prayer of", "Prayer:");
					name = string.gsub(name, "Lesser", "L.");
					name = string.gsub(name, "Greater", "G.");
					name = string.gsub(name, "Aspect of the", "Aspect:");					
				end
				
				if (string.len(name) > options.maxlength) then
					name = string.sub(name, 0, options.maxlength);
					name = name .. "..";
					return name;
				else
					return name;
				end
			end
		end
	end
	return nil;
end

function mbb_GetBuffType(unit, i, filter)
	if (filter == nil) then
		filter = "HELPFUL|HARMFUL";
	end
	getglobal("mbbTooltipTextLeft1"):SetText("");
	getglobal("mbbTooltipTextRight1"):SetText("");
	getglobal(this:GetName().."buffTypeText"):SetText("");

	local buffIndex, untilCancelled = GetPlayerBuff(this:GetID(), filter);
	--local buffIndex, untilCancelled = GetPlayerBuff(i, "HARMFUL");

	local buff;

	if (buffIndex < 24) then
		buff = buffIndex;
		if (buff == -1) then
			buff = nil;
		end
	end

	if (buff) then
		local tooltip = mbbTooltip;
		tooltip:SetOwner(UIParent, "ANCHOR_NONE");
		if (unit == "player" and tooltip ) then
			tooltip:SetPlayerBuff(buffIndex);
		end
		local toolTipText = getglobal("mbbTooltipTextRight1");
		if (toolTipText) then
			local name = toolTipText:GetText();
			if ( name ~= nil ) then
				return name;
			end
		end
	end
	return nil;
end

function Gypsy_GetStringTime(seconds)
	local duration = "";
	local minutes = floor(seconds / 60);
	local secsleft = seconds - (60 * minutes);

	if (seconds >= 60) then
		secsleft = floor(secsleft);
		duration = minutes .."m " .. secsleft .. "s";
	else
		if (secsleft < 1) then
			duration = string.sub(secsleft, 2, 3);
			duration = " ".. duration .. "s";
			return duration;
		else
			duration = floor(seconds) .. " second";
			if(seconds >= 2) then
				duration = duration .. "s";
			end
		end
	end
	return duration;
end



function MBB_LoadGypsySetup()
	local args = {};
	mbb_showdispeltypemodes = {};

	-- Radio button properties
	mbb_showdispeltypemodes[1] = {
		["setting"] = "greenred";
		["label"] = "Green if Dispellable, Red otherwise";
		["tooltip"] = "This mode shows the Debuff Types \nin Green if you can Dispel it.\nRed if you cannot.";
	};
	mbb_showdispeltypemodes[2] = {
		["setting"] = "greenyellow";
		["label"] = "Green if Dispellable, Yellow otherwise";
		["tooltip"] = "This mode shows the Debuff Types \nin Green if you can Dispel it.\nYellow if you cannot.";
	};


	args.name = "mbb_category";
	args.save = "MBB_OPTIONS";
	args.usechar = "1";
	args.char = player;
	args.label = "BuffBar";
	args.tooltip = "GypsyMod BuffBar - Reborn v" .. options.version;
	Gypsy_OmniAddCategory(args);

	args = {};
	args.name = "mbb_pageheadline";
	args.parent = "mbb_category";
	args.disptype = "options";
	args.regtype = "headline";
	args.label1 = "GypsyMod BuffBar - Reborn v" .. options.version;
	Gypsy_OmniRegister(args);

	args = {};
	args.name = "mbb_enabled";
	args.parent = "mbb_category";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "enabled";
	args.label1 = "Enable";
	Gypsy_OmniRegister(args);

	if (GYPSY_HOTBAR == nil) then
		args = {};
		args.name = "mbb_checkbuttonlockunlock";
		args.parent = "mbb_category";
		args.depend = "mbb_enabled";
		args.disptype = "options";
		args.regtype = "checkbutton";
		args.var = "lockbuffbar";
		args.label1 = "Lock/Unlock BuffBar";
		Gypsy_OmniRegister(args);
	end

	args = {};
	args.name = "mbb_buttonheader2";
	args.regtype = "header";

	args.disptype = "options";
	args.parent = "mbb_category";
	args.label = "Display Options";
	Gypsy_OmniRegister(args);
	


	--SHOW DEBUFFS ONLY
	args = {};
	args.name = "mbb_checkbuttondisplaymode";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "displaymode";
	args.label1 = "Show GypsyMod Style Debuffs Only";
	args.tooltip = "Displays defaultui helpful Buffs \nand GypsyMod Style harmful Debuffs";
	Gypsy_OmniRegister(args);
	
	
	--POPULATE FROM BOTTOM TO TOP
	args = {};
	args.name = "mbb_checkbuttongrowbuffsup";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "growbuffsup";
	args.label1 = "Populate Buffs from Bottom to Top";
	args.tooltip = "This mode populates the BuffS Upwards, from the Bottom to Top";
	Gypsy_OmniRegister(args);

	--ICONMODE
	args = {};
	args.name = "mbb_checkbuttoniconmode";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "iconmode";
	args.label1 = "Icons on Right";
	args.tooltip = "When checked Buff Icons appear on the Right side, \nWhen unchecked on the Left";
	Gypsy_OmniRegister(args);

	--BUFFBAR SCALE
	args = {};
	args.name = "mbb_scale";
	args.regtype = "slider";
	args.disptype = "options";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.var = "scale";
	args.func = "Gypsy_BuffFrameCapsuleOnUpdate";
	args.slidermin = 60;
	args.slidermax = 120;
	args.sliderstep = 1;
	args.sliderunit = "%";
	args.label1 = "BuffBar Scale";
	args.tooltip = "Set the buffbar from 60 to 120% \nit's original scale";
	Gypsy_OmniRegister(args);


	--BLACK BACKDROP?
	args = {};
	args.name = "mbb_checkbuttonblackbackdrop";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "blackbackdrop";
	args.label1 = "Show Black Buff BackGrounds";
	args.tooltip = "Displays BuffBar with Black Background \ninstead of Blue or Red";
	Gypsy_OmniRegister(args);


	--BACKDROP TRANSPARENCY
	args = {};
	args.name = "mbb_backdroptransparency";
	args.regtype = "slider";
	args.disptype = "options";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.var = "backdroptransparency";
	args.func = "Gypsy_BuffFrameCapsuleOnUpdate";
	args.slidermin = 0;
	args.slidermax = 100;
	args.sliderstep = 1;
	args.sliderunit = "%";
	args.label1 = "BackGround Transparency";
	args.tooltip = "Set BackGround Transparency from 0% to 100%";
	Gypsy_OmniRegister(args);

	--flash
	args = {};
	args.name = "mbb_warningtimeslider";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.disptype = "options";
	args.regtype = "slider";
	args.var = "flashtime";
	args.slidermin = 0;
	args.slidermax = 30;
	args.sliderstep = 1;
	args.sliderunit = "s";
	args.label1 = "Flash Buffs when Duration is below X Seconds";
	args.tooltip = "Flash buffs when between 1-30 seconds, \nsetting of 0 disables flashing";
	Gypsy_OmniRegister(args);

	--SHOW DEBUFFTYPES
	args = {};
	args.name = "mbb_checkbuttonshowdebufftypes";
	args.parent = "mbb_category";
	args.depend = "mbb_enabled";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "showdebufftypes";
	args.label1 = "Show Debuff Types";
	args.tooltip = "Displays debuff types such as:\nMagic, Disease, Poison, or Curses";
	Gypsy_OmniRegister(args);


	--highlight dispellable 
	args = {};
	args.name = "mbb_checkbuttonshowdispellable";
	args.parent = "mbb_category";
	args.depend = "mbb_checkbuttonshowdebufftypes";
	args.disptype = "options";
	args.regtype = "checkbutton";
	args.func = "Gypsy_BuffFrameUpdateAll";
	args.var = "showdispellable";
	args.label1 = "Highlight Debuff Types";
	args.tooltip = "Allows you to change the text color of debuff types based on if you can Dispel it or not";
	Gypsy_OmniRegister(args);

	--debufftype text color radio
	args = {};
	args.name = "mbb_showdispeltypemodes";
	args.regtype = "radiobutton";
	args.disptype = "options";
	args.parent = "mbb_category";
	args.depend = "mbb_checkbuttonshowdispellable";
	args.var = "debufftypetextcolor";
	args.settings = "mbb_showdispeltypemodes";
	args.func = "Gypsy_BuffFrameUpdateAll";
	Gypsy_OmniRegister(args);
end


function mbb_SlashCommands(msg)
	local tempnum = 0;
	msg = string.lower(msg);
	local words = {};
	for word in string.gfind(msg, "%w+") do
		table.insert(words, word);
	end

	if (msg == "") then
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] - Current Options");
		DEFAULT_CHAT_FRAME:AddMessage("--------------------------------------------------------");
		DEFAULT_CHAT_FRAME:AddMessage("....version                : " .. options.version);
		DEFAULT_CHAT_FRAME:AddMessage("....enabled               : " .. options.enabled .. "   yes=1 no=0");
		DEFAULT_CHAT_FRAME:AddMessage("....flashtime     (0-30): " .. options.flashtime .. "   flash when under X secs");
		DEFAULT_CHAT_FRAME:AddMessage("....lockbuffbar          : " .. options.lockbuffbar .."   yes=1 no=0");
		DEFAULT_CHAT_FRAME:AddMessage("....showduration       : " .. options.showduration .. "   yes=1 no=0");
		DEFAULT_CHAT_FRAME:AddMessage("....showdebufftypes  : " .. options.showdebufftypes .. "   yes=1 no=0");
		DEFAULT_CHAT_FRAME:AddMessage("....iconmode            : " .. options.iconmode .. "   left=0 right=1 use /mmb iconswap");
		DEFAULT_CHAT_FRAME:AddMessage("....displaymode        : " .. options.displaymode .. "   default=0 Show Debuffs Only=1");
		DEFAULT_CHAT_FRAME:AddMessage("....growbuffsup        : " .. options.growbuffsup .. " yes=1 no=0(populate from bottom to top)");
		DEFAULT_CHAT_FRAME:AddMessage("....scale (60-120)    : " .. options.scale .. " size buffbar from 60% to 120% orig size");
		DEFAULT_CHAT_FRAME:AddMessage("....maxlength       : " .. options.maxlength .. "  (max length of names) default=24");
		DEFAULT_CHAT_FRAME:AddMessage("....showdispellable        : " .. options.showdispellable .. " Highlight debuff types, 1=yes 0=no");
		DEFAULT_CHAT_FRAME:AddMessage("....debufftypetextcolor  : " .. options.debufftypetextcolor .. "  greenred or greenyellow");
		DEFAULT_CHAT_FRAME:AddMessage("....backdroptransparency: " .. options.backdroptransparency .. "  Background transparency, 0-100");
		DEFAULT_CHAT_FRAME:AddMessage("....subnames: " .. options.subnames .. " Substitute words in spellnames 0=no 1=yes");
		DEFAULT_CHAT_FRAME:AddMessage("---------------------------------------------------------");
		DEFAULT_CHAT_FRAME:AddMessage("/mbb set <optionname> <value>");
		DEFAULT_CHAT_FRAME:AddMessage("To Change between display modes\n (GypsyMod Buffs|Debuffs or\nGypsyMod Debuffs Only)");
		DEFAULT_CHAT_FRAME:AddMessage("/mbb displaymode");
		DEFAULT_CHAT_FRAME:AddMessage("/mbb iconswap     To change the icons position");
		DEFAULT_CHAT_FRAME:AddMessage("/bblock                 To Unlock/Lock the BuffBar");
		DEFAULT_CHAT_FRAME:AddMessage("---------------------------------------------------------");

	elseif (msg == "reset") then

		mbb_reset();
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Options reset to defaults.");
		Gypsy_BuffFrameUpdateAll();
		Gypsy_BuffFrameCapsuleOnUpdate();
		MBB_LoadGypsySetup();

	elseif (words[1] == "set") then
		if (words[2] ~= nil) then

			if (words[3] ~= nil) then
				words[3] = tonumber(words[3]);
			end

			if(words[2] == "enabled") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.enabled = words[3];
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Enabled set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for enabled setting");
				end
			elseif (words[2] == "flashtime") then
				if (words[3] ~= nil and words[3] <= 30) then
					options.flashtime = words[3];
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] flash warning time set to " .. words[3] .. " seconds or less");
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for flashtime, must be 0-30");
				end
			elseif (words[2] == "lockbuffbar") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.lockbuffbar = words[3];
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] lockbuffbar set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for lockbuffbar, must be 0 or 1");
				end
			elseif (words[2] == "showduration") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.showduration = words[3];
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] showduration of buffs set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for showduration");
				end
			elseif (words[2] == "showdebufftypes") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.showdebufftypes = words[3];
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] showdebufftypes set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for showdebufftypes");
				end
			elseif (words[2] == "iconmode") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.iconmode = words[3];
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] iconmode set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for iconmode");
				end
			elseif (words[2] == "displaymode") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.displaymode = words[3];
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] displaymode set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for displaymode");
				end
			elseif (words[2] == "growbuffsup") then
				if (words[3] ~= nil and words[3] <= 1) then
					options.growbuffsup = words[3];
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] growbuffsup set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for growbuffsup");
				end
			--elseif (words[2] == "paddingup") then
			--	if (words[3] ~= nil and words[3] > 20) then
			--		options.paddingup = words[3];
			--		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] paddingup set to " .. words[3]);
			--	else
			--		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for paddingup");
			--	end
			--elseif (words[2] == "paddingdown") then
			--	if (words[3] ~= nil) then
			--		options.paddingdown = words[3];
			--		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] paddingdown set to " .. words[3]);
			--	else
			--		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for paddingdown");
			--	end
			elseif (words[2] == "scale") then
				tempnum = tonumber(words[3]);
				if (tempnum >= 60 and tempnum <= 120) then
					options.scale = tempnum;
					Gypsy_BuffFrameCapsuleOnUpdate();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] scale set to " .. words[3] .. "% its original size");
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for scale, must be from 60-120");
				end
			elseif (words[2] == "maxlength") then
				tempnum = tonumber(words[3]);
				if (tempnum ~= nil) then
					options.maxlength = tempnum;
					Gypsy_BuffFrameCapsuleOnUpdate();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] maxlength of names set to " .. words[3] .. " characters");
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for maxlength");
				end
			elseif (words[2] == "version") then
				tempnum = tonumber(words[3]);
				if (tempnum ~= nil) then
					options.version = tempnum;
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] version set to " .. words[3]);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Invalid numeric value for version");
				end
			elseif (words[2] == "backdroptransparency") then
				tempnum = tonumber(words[3]);
				if( tempnum <= 100) then
					options.backdroptransparency = tempnum;
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] backdroptransparency set to " .. options.backdroptransparency);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] backdroptransparency range is from 0 to 100");
				end
			elseif (words[2] == "subnames") then
				tempnum = tonumber(words[3]);
				if( tempnum <= 1) then
					options.subnames = tempnum;
					Gypsy_BuffFrameUpdateAll();
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] subnames set to " .. options.subnames);
				else
					DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] subnames can only be set to 0 or 1");
				end
			else
				DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] invalid option: " .. words[2]);
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] you must specify a command");
		end
	elseif (words[1] == "iconswap") then
		if (options.iconmode == 0) then
			options.iconmode = 1;
			Gypsy_BuffFrameUpdateAll();
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Icons now positioned on the RIGHT side of bar.");
		else
			options.iconmode = 0;
			Gypsy_BuffFrameUpdateAll();
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Icons now positioned on the LEFT side of bar.");
		end
	elseif (words[1] == "displaymode") then
		if (options.displaymode == 0) then
			options.displaymode = 1;
			Gypsy_BuffFrameUpdateAll();
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Now showing Gypsymod style DEBUFFS ONLY.");
		else
			options.displaymode = 0;
			Gypsy_BuffFrameUpdateAll();
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Now showing Gypsymod style BUFFS & DEBUFFS.");
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] invalid command: ");
	end
end


-- ** SLASH HANDLER FUNCTIONS ** --
function Gypsy_ShowBuffBarSlashHandler(msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		options.enabled = 1;
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Showing GypsyMod buff frame.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		options.enabled = 0;
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Showing default buff frame.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		options.enabled = Gypsy_DefaultShowBuffFrame;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting buff frame state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /buffbarenable /bbenable", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Enable the GypsyMod BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Enable the default buff display.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default option state.", 1, 1, 1);
	elseif (msg == "") then
		if (options.enabled == 1) then
			options.enabled = 0;
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Showing default buff frame.", 1, 1, 1);
		else
			options.enabled = 1;
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Showing GypsyMod buff frame.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /buffbarenable /bbenable", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Enable the GypsyMod BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Enable the default buff display.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default option state.", 1, 1, 1);
	end
	Gypsy_BuffFrameUpdateAll();
end

function Gypsy_LockBuffBarSlashHandler(msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "yes" or msg == "1" or msg == "true") then
		--Gypsy_LockBuffBar = 1;
		options.lockbuffbar = 1;
		Gypsy_LockBuffBar = options.lockbuffbar;
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Locking buff frame.", 1, 1, 1);
	elseif (msg == "no" or msg == "0" or msg == "false") then
		options.lockbuffbar = 0;
		Gypsy_LockBuffBar = options.lockbuffbar;
		DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Unlocking buff frame.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		options.lockbuffbar = Gypsy_DefaultLockBuffBar;
		Gypsy_LockBuffBar = options.lockbuffbar;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting buff frame lock state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /buffbarlock /bblock", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Lock the GypsyMod BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Unlock the GypsyMod BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default option state.", 1, 1, 1);
	elseif (msg == "") then
		if (options.lockbuffbar == 1) then
			options.lockbuffbar = 0;
			Gypsy_LockBuffBar = options.lockbuffbar;
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Unlocking buff frame.", 1, 1, 1);
		else
			options.lockbuffbar = 1;
			Gypsy_LockBuffBar = options.lockbuffbar;
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Locking buff frame.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /buffbarlock /bblock", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Lock the GypsyMod BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Unlock the GypsyMod BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default option state.", 1, 1, 1);
	end
end

function Gypsy_ShowDurationSlashHandler(msg)
	-- Due to the nature of slash commands, 'msg' will never be nil, so we don't require a check
	msg = string.lower(msg);
	if (msg == "show" or msg == "1" or msg == "true") then
		options.showduration = 1;
		DEFAULT_CHAT_FRAME:AddMessage("Showing effect duration text.", 1, 1, 1);
	elseif (msg == "hide" or msg == "0" or msg == "false") then
		options.showduration = 0;
		DEFAULT_CHAT_FRAME:AddMessage("Hiding effect duration text.", 1, 1, 1);
	elseif (msg == "default" or msg == "reset" or msg == "revert") then
		options.showduration = Gypsy_DefaultShowDuration;
		DEFAULT_CHAT_FRAME:AddMessage("Reverting effect duration text state to default.", 1, 1, 1);
	elseif (msg == "help") then
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /buffbarshowduration /bbshowduration", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show duration text labels on the BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide duration text.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default option state.", 1, 1, 1);
	elseif (msg == "") then
		if (options.showduration == 1) then 
			options.showduration = 0;
			Gypsy_ShowDuration = options.showduration; 
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Hiding effect duration text.", 1, 1, 1);
		else 
			options.showduration = 1;
			Gypsy_ShowDuration = options.showduration; 
			DEFAULT_CHAT_FRAME:AddMessage("[BUFFBAR] Showing effect duration text.", 1, 1, 1);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("Valid parameters for: /buffbarshowduration /bbshowduration", 1, 0.89, 0.01);
		DEFAULT_CHAT_FRAME:AddMessage("   help - This listing.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   show, true, or 1 - Show duration text labels on the BuffBar.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   hide, false, or 0 - Hide duration text.", 1, 1, 1);
		DEFAULT_CHAT_FRAME:AddMessage("   default, reset, or revert - Revert to default option state.", 1, 1, 1);
	end
	Gypsy_BuffFrameUpdateAll();
end

function Gypsy_ResetBuffBarSlashHandler(msg)
	Gypsy_BuffFrameReset();
	DEFAULT_CHAT_FRAME:AddMessage("Resetting all BuffBar parameters to default.", 1, 1, 1);
end

function Gypsy_BuffBarSlashHandler(msg)
	DEFAULT_CHAT_FRAME:AddMessage("Valid slash commands for the BuffBar add-on", 1, 0.89, 0.01);
	DEFAULT_CHAT_FRAME:AddMessage("All commands may begin with either /buffbar or /bb, ex. /buffbarenable", 1, 0.80, 0.01);
	DEFAULT_CHAT_FRAME:AddMessage("Entering help after any command will display a list of valid parameters.", 1, 0.80, 0);
	DEFAULT_CHAT_FRAME:AddMessage("   enable - Enable/Disable the BuffBar.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   lock - Lock/Unlock the BuffBar.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   showduration - Show/Hide effect duration text.", 1, 1, 1);
	DEFAULT_CHAT_FRAME:AddMessage("   reset - Reset buff frame state.", 1, 1, 1);
end
